/*
sudoku - Create and play sudoku

Copyright (C) 2007 Sven Heinzel <Sven.Heinzel@gmx.net>
 
This program is free software; you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation; either version 2 of the License, or
(at your option) any later version.
 
This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.
 
You should have received a copy of the GNU General Public License
along with this program; if not, write to the Free Software
Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
*/

#pragma once

#ifndef DISPLAYOPTIONS_H
#define DISPLAYOPTIONS_H

#include "sudoku_gui.h"

class DisplayOptions
{
public:
    enum Show
    {
        ShowFixed             = 0x00000001,
        ShowNonFixed          = 0x00000002,
        ShowActive            = 0x00000004,
        ShowAllowed           = 0x00000008,
        ShowTips              = 0x00000010,
        ShowSimplifiedAllowed = 0x00000020,

        ShowFields = ShowFixed | ShowNonFixed,
        ShowHelp   = ShowAllowed | ShowTips | ShowSimplifiedAllowed,
        ShowAll    = 0x0000002f
    };

    DisplayOptions()
        : mShow( ShowFields )
    {
    }

    DisplayOptions( int  show )
        : mShow( show )
    {
    }

    void SetVisible( int  show, bool  visible = true )
    {
        if( visible ) mShow |= show;
        else          mShow &= ~show;
    }

    void ToggleVisible( Show  show )
    {
        SetVisible( show, ! IsVisible( show ) );
    }

    bool IsVisible( Show  show ) const
    {
        return ( mShow & show ) != 0;
    }

    bool operator==( DisplayOptions const &rOther )
    {
        return mShow == rOther . mShow;
    }

    bool operator!=( DisplayOptions const &rOther )
    {
        return !operator==( rOther );
    }

private:
    int  mShow;
};

#endif
