/*
sudoku - Create and play sudoku

Copyright (C) 2007 Sven Heinzel <Sven.Heinzel@gmx.net>
 
This program is free software; you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation; either version 2 of the License, or
(at your option) any later version.
 
This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.
 
You should have received a copy of the GNU General Public License
along with this program; if not, write to the Free Software
Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
*/

#pragma once

#ifndef DOCKWIDGETS_H
#define DOCKWIDGETS_H

#include "sudoku_gui.h"
#include "GridView.h"
#include "GenerateWidget.h"

class BaseDockWidget : public QWidget
{
    Q_OBJECT

public:
    explicit BaseDockWidget( QWidget  *pParent = 0, Qt::WindowFlags  flags = 0 )
        : QWidget( pParent, flags )
        , mpGridView( 0 )
    {
    }

    virtual ~BaseDockWidget() {}

public Q_SLOTS:
    void SetGridView( GridView  *pGridView )
    {
        // Disconnect all old signals
        if( mpGridView ) disconnect( mpGridView, 0, 0, 0 );

        // Set new grid view and set widgets
        mpGridView = pGridView;
        if( mpGridView ) connect( mpGridView, SIGNAL( Modified() ), this, SLOT( UpdateDisplay() ) );
        setEnabled( !!mpGridView );
        UpdateDisplay();
    }

protected Q_SLOTS:
    virtual void UpdateDisplay() {}

protected:
    GridView  *mpGridView;
};

class GenerateDockWidget : public BaseDockWidget
{
    Q_OBJECT

public:
    explicit GenerateDockWidget( GridView  *pGridView, QWidget  *pParent = 0, Qt::WindowFlags  flags = 0 );
    virtual ~GenerateDockWidget() {}

private Q_SLOTS:
    void Generate();

private:
    GenerateWidget  *mpGenerateWidget;
};

class SolveDockWidget : public BaseDockWidget
{
    Q_OBJECT

public:
    explicit SolveDockWidget( GridView  *pGridView, QWidget  *pParent = 0, Qt::WindowFlags  flags = 0 );
    virtual ~SolveDockWidget() {}

protected Q_SLOTS:
    virtual void UpdateDisplay();

private Q_SLOTS:
    void ChangeDisplay();
    void SolveOne();
    void SolveAll();

private:
    QCheckBox  *mpCheckShowAllowed;
    QCheckBox  *mpCheckShowSimplifiedAllowed;
    QCheckBox  *mpCheckShowTips;
};

class AnalyseDockWidget : public BaseDockWidget
{
    Q_OBJECT

public:
    explicit AnalyseDockWidget( GridView  *pGridView, QWidget  *pParent = 0, Qt::WindowFlags  flags = 0 );
    virtual ~AnalyseDockWidget() {}

protected Q_SLOTS:
    virtual void UpdateDisplay();

private:
    QLabel  *mpLabelSolution;
    QLabel  *mpLabelPairs;
    QLabel  *mpLabelBacktrack;
};

#endif
