/*
sudoku - Create and play sudoku

Copyright (C) 2007 Sven Heinzel <Sven.Heinzel@gmx.net>
 
This program is free software; you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation; either version 2 of the License, or
(at your option) any later version.
 
This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.
 
You should have received a copy of the GNU General Public License
along with this program; if not, write to the Free Software
Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
*/

#pragma once

#ifndef EXPORTOPTIONS_H
#define EXPORTOPTIONS_H

#include "sudoku_gui.h"
#include "DisplayOptions.h"

class ExportOptions
{
public:
    enum Unit
    {
        Millimeter = 0,
        Centimeter,
        Inch
    };

    ExportOptions()
        : mFormat( "PNG" )
        , mDPI( 300 )
        , mCanvasEnabled( false )
        , mCanvasWidth( 210.0 )
        , mCanvasHeight( 297.0 )
        , mGridSize( 100.0 )
        , mAlignment( Qt::AlignCenter )
        , mDisplayOptions( DisplayOptions::ShowFixed )
        , mFileName()
    {
    }

    ExportOptions( std::string const  &rFormat, int  dpi = 300, bool  canvasEnabled = false, double  canvasWidth = 210.0, double  canvasHeight = 297.0, double  gridSize = 100.0, Qt::Alignment  alignment = Qt::AlignCenter, DisplayOptions const  &rDisplayOptions = DisplayOptions( DisplayOptions::ShowFixed ), std::string const  &rFileName = std::string() )
        : mFormat( rFormat )
        , mDPI( dpi )
        , mCanvasEnabled( canvasEnabled )
        , mCanvasWidth( canvasWidth )
        , mCanvasHeight( canvasHeight )
        , mGridSize( gridSize )
        , mAlignment( alignment )
        , mDisplayOptions( rDisplayOptions )
        , mFileName( rFileName )
    {
    }

    /*
    void SetType( Type  type )
    {
        mType = type;
    }

    void SetFixedFields( int  fixedFields )
    {
        mFixedFields = ValidFixedField( fixedFields );
    }

    void SetSymmetric( bool  symmetric )
    {
        mSymmetric = symmetric;
    }

    void SetAllowBacktrack( bool  allowBacktrack )
    {
        mAllowBacktrack = allowBacktrack;
    }

    int GetType() const
    {
        return mType;
    }

    int GetFixedFields() const
    {
        return mFixedFields;
    }

    static int GetFixedFieldsMin()
    {
        return 24;
    }

    static int GetFixedFieldsMax()
    {
        return 45;
    }

    bool IsSymmetric() const
    {
        return mSymmetric;
    }

    bool IsAllowBacktrack() const
    {
        return  mAllowBacktrack;
    }
    */

private:
    std::string     mFormat;
    int             mDPI;
    bool            mCanvasEnabled;
    double          mCanvasWidth;
    double          mCanvasHeight;
    double          mGridSize;
    Qt::Alignment   mAlignment;
    DisplayOptions  mDisplayOptions;
    std::string     mFileName;
};

#endif
