/*
sudoku - Create and play sudoku

Copyright (C) 2007 Sven Heinzel <Sven.Heinzel@gmx.net>
 
This program is free software; you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation; either version 2 of the License, or
(at your option) any later version.
 
This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.
 
You should have received a copy of the GNU General Public License
along with this program; if not, write to the Free Software
Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
*/

#pragma once

#ifndef GENERATEWIDGET_H
#define GENERATEWIDGET_H

#include "sudoku_gui.h"

#include "sudoku/GenerateOptions.h"

class GenerateWidget : public QWidget
{
    Q_OBJECT

public:
    explicit GenerateWidget( Sudoku::GenerateOptions const  &rOptions, QWidget  *pParent = 0, Qt::WindowFlags  flags = 0 );
    virtual ~GenerateWidget();

    void SetGenerateOptions( Sudoku::GenerateOptions const  &rOptions )
    {
        mpComboType -> setCurrentIndex( mpComboType -> findData( rOptions . GetType() ) );
        mpSpinCount -> setValue( rOptions . GetFixedFields() );
        mpCheckSymmetric -> setChecked( rOptions . IsSymmetric() );
        mpSpinPairs -> setValue( rOptions . GetMaxPairSolving() );
        mpCheckBacktrack -> setChecked( rOptions . IsBacktrackAllowed() );
    }

    Sudoku::GenerateOptions GetGenerateOptions() const
    {
        return Sudoku::GenerateOptions( static_cast< Sudoku::GenerateOptions::Type >( mpComboType -> itemData( mpComboType -> currentIndex() ) . toInt() ),
                                        mpSpinCount -> value(),
                                        mpCheckSymmetric -> isChecked(),
                                        Sudoku::SolveOptions( mpSpinPairs -> value(),
                                                              mpCheckBacktrack -> isChecked() ) );
    }

private slots:
    void TypeChanged( int  index );

private:
    QComboBox  *mpComboType;
    QGroupBox  *mpGroupFields;
    QSpinBox   *mpSpinCount;
    QCheckBox  *mpCheckSymmetric;
    QGroupBox  *mpGroupSolve;
    QSpinBox   *mpSpinPairs;
    QCheckBox  *mpCheckBacktrack;
};

#endif
