/*
sudoku - Create and play sudoku

Copyright (C) 2007 Sven Heinzel <Sven.Heinzel@gmx.net>
 
This program is free software; you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation; either version 2 of the License, or
(at your option) any later version.
 
This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.
 
You should have received a copy of the GNU General Public License
along with this program; if not, write to the Free Software
Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
*/

#pragma once

#ifndef MAINWINDOW_H
#define MAINWINDOW_H

#include "sudoku_gui.h"
#include "GridView.h"
#include "GenerateWidget.h"

class MainWindow : public QMainWindow
{
    Q_OBJECT

public:
    explicit MainWindow( QWidget  *pParent = 0, Qt::WindowFlags  flags = 0 );
    virtual ~MainWindow();

    static QString ConfigPath()
    {
        return QDir::homePath() + "/.sternenwanderer.net/Sudoku";
    }

private:
    void Init();
    void InitActions();
    void InitMenus();
    void InitToolBars();
    void InitDockWidgets();
    void InitWidgets();

    void SetupAction( QString const  &rKey, QString const  &rText, QIcon const  &rIcon, bool const  isCheckable, QKeySequence const  &rShortcut, QString const  &rStatusTip, QObject const  *pTarget, char const  *pMethod );
    void SetupMenu( QString const  &rKey, QString const  &rTitle );
    void SetupToolBar( QString const  &rKey, QString const  &rTitle );
    void SetupDockWidget( QString const  &rKey, QString const  &rTitle, QWidget  *pWidget );

    bool SaveDoc( QDomDocument const  &rDoc, QString const  &rFileName, QString  * const pError = 0 ) const;
    bool LoadDoc( QDomDocument  &rDoc, QString const  &rFileName, QString  * const pError = 0 ) const;

    bool SaveConfig();
    bool LoadConfig();

    bool Save( QDomElement  &rParent ) const;
    bool Load( QDomElement const  &rParent );

    bool SudokuSave( QString const  &rFileName );
    bool SudokuLoad( QString const  &rFileName );

    QHash< QString, QAction* >      mActions;
    QHash< QString, QMenu* >        mMenus;
    QHash< QString, QToolBar* >     mToolBars;
    QHash< QString, QDockWidget* >  mDockWidgets;
    QMenuBar           *mpMenuBar;
    QLabel             *mpLabelProgress;
    GridView           *mpGridView;
    QMenu              *mpMenuToolbars;
    QList< QAction* >  mRecentList;
    QSignalMapper      mRecentMapper;
    QTranslator        mQtTrans;
    QTranslator        mSudokuTrans;
    QByteArray         mState;
    int                mStateVersion;
    QStringList        mRecentFiles;
    QPrinter           mPrinter;
    Sudoku::GenerateOptions  mGenerateOptions;
    QString            mFileName;
    QString            mLastPath;

private Q_SLOTS:
    void About();
    void AboutQt();

    void UpdateMenuRecent();
    void UpdateState();
    void AddRecentFile( QString const  &rFile );
    void DrawPrint( QPrinter  *pPrinter );

    void SudokuRestart();
    void SudokuGenerate();
    void SudokuGenerate( Sudoku::GenerateOptions const  &rOptions );
    void SudokuOpen( QString const  &rFile = QString() );
    void SudokuSave();
    void SudokuSaveAs();
    void SudokuExport();
    void SudokuPrint();
    void SudokuPrinterSetup();
    void SudokuUndo();
    void SudokuRedo();
    void SudokuToggleAllowedNumbers();
    void SudokuToggleSimplifiedAllowedNumbers();
    void SudokuToggleSolutionTips();
    void SudokuSolveOne();
    void SudokuSolveAll();
};

#endif
