/*
sudoku - Create and play sudoku

Copyright (C) 2007 Sven Heinzel <Sven.Heinzel@gmx.net>
 
This program is free software; you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation; either version 2 of the License, or
(at your option) any later version.
 
This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.
 
You should have received a copy of the GNU General Public License
along with this program; if not, write to the Free Software
Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
*/

#include "sudoku_gui.h"
#include "GenerateWidget.h"

GenerateWidget::GenerateWidget(
    Sudoku::GenerateOptions const  &rOptions,
    QWidget                        *pParent,
    Qt::WindowFlags                flags
)
    : QWidget( pParent, flags )
    , mpComboType( 0 )
    , mpGroupFields( 0 )
    , mpSpinCount( 0 )
    , mpCheckSymmetric( 0 )
    , mpSpinPairs( 0 )
    , mpCheckBacktrack( 0 )
{
    mpComboType = new QComboBox( this );
    mpComboType -> addItem( tr( "Empty sudoku" ), Sudoku::GenerateOptions::GenerateEmpty );
    mpComboType -> addItem( tr( "Generate fields" ), Sudoku::GenerateOptions::GenerateFields );
    //mpComboType -> addItem( tr( "Fill fixed fields" ), Sudoku::GenerateOptions::FillFixedFields );
    connect( mpComboType, SIGNAL( currentIndexChanged( int ) ), SLOT( TypeChanged( int ) ) );

    // Fields
    mpGroupFields = new QGroupBox( tr( "Fields" ), this );
    QLabel  *p_label_count = new QLabel( tr( "Minimum number" ) );
    mpSpinCount = new QSpinBox( mpGroupFields );
    mpSpinCount -> setRange( Sudoku::GenerateOptions::GetFixedFieldsMin(), Sudoku::GenerateOptions::GetFixedFieldsMax() );
    mpCheckSymmetric = new QCheckBox( tr( "Symmetric" ), mpGroupFields );

    QGridLayout  *p_grid_layout_fields = new QGridLayout( mpGroupFields );
    p_grid_layout_fields -> addWidget( p_label_count, 0, 0 );
    p_grid_layout_fields -> addWidget( mpSpinCount, 0, 1 );
    p_grid_layout_fields -> addWidget( mpCheckSymmetric, 1, 0, 1, 2 );
    p_grid_layout_fields -> setColumnStretch( 1, 1 );

    // Solve options
    mpGroupSolve = new QGroupBox( tr( "Solve options" ), this );
    QLabel  *p_label_pair = new QLabel( tr( "Maximum pair" ) );
    mpSpinPairs = new QSpinBox( mpGroupSolve );
    mpSpinPairs -> setRange( 1, 7 );
    mpSpinPairs -> setSpecialValueText( tr( "None" ) );
    mpCheckBacktrack = new QCheckBox( tr( "Backtrack" ), mpGroupSolve );

    QGridLayout  *p_grid_layout_solve = new QGridLayout( mpGroupSolve );
    p_grid_layout_solve -> addWidget( p_label_pair, 0, 0 );
    p_grid_layout_solve -> addWidget( mpSpinPairs, 0, 1 );
    p_grid_layout_solve -> addWidget( mpCheckBacktrack, 1, 0, 1, 2 );
    p_grid_layout_solve -> setColumnStretch( 1, 1 );

    // Main layout
    QVBoxLayout  *p_vbox_layout = new QVBoxLayout( this );
    p_vbox_layout -> setContentsMargins( 0, 0, 0, 0 );
    p_vbox_layout -> addWidget( mpComboType );
    p_vbox_layout -> addWidget( mpGroupFields );
    p_vbox_layout -> addWidget( mpGroupSolve );

    // Set initial values
    SetGenerateOptions( rOptions );
    TypeChanged( mpComboType -> currentIndex() );
}

GenerateWidget::~GenerateWidget()
{
}

void GenerateWidget::TypeChanged(
    int  index
)
{
    int const  type = mpComboType -> itemData( index ) . toInt();
    mpGroupFields -> setEnabled( type == Sudoku::GenerateOptions::GenerateFields );
    mpGroupSolve -> setEnabled( type == Sudoku::GenerateOptions::GenerateFields );
}
