;sudoku - Create and play sudoku
;
;Copyright (C) 2007 Sven Heinzel <Sven.Heinzel@gmx.net>
;
;This program is free software; you can redistribute it and/or modify
;it under the terms of the GNU General Public License as published by
;the Free Software Foundation; either version 2 of the License, or
;(at your option) any later version.
;
;This program is distributed in the hope that it will be useful,
;but WITHOUT ANY WARRANTY; without even the implied warranty of
;MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
;GNU General Public License for more details.
;
;You should have received a copy of the GNU General Public License
;along with this program; if not, write to the Free Software
;Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
;
;Start Menu Folder Selection Example Script
;Written by Joost Verburg

;--------------------------------
; Include Modern UI
;--------------------------------

!include "MUI.nsh"

;--------------------------------
; General
;--------------------------------

; Settings

!define REG_ROOT "HKLM"
!define REG_KEY_BASE "Software\sternenwanderer.net"
!define REG_KEY "${REG_KEY_BASE}\sudoku"
!define REG_KEY_UNINSTALL "Software\Microsoft\Windows\CurrentVersion\Uninstall\net.sternenwanderer.sudoku"
!define UNINST_NAME "uninst-sudoku.exe"

; Name and file

Name "Sudoku"

; Default installation folder

InstallDir "$PROGRAMFILES\sudoku"

; Get installation folder from registry if available

InstallDirRegKey ${REG_ROOT} "${REG_KEY}" ""

; Trick to sign uninstaller

!ifdef INNER
  !echo "Just creating the uninstaller"     ; just to see what's going on
  OutFile "$%TEMP%\tempinstaller.exe"       ; not really important where this is
  SetCompress off                           ; for speed
!else
  !echo "Normal invocation"
 
  ; Call makensis again, defining INNER.  This writes an installer for us which, when
  ; it is invoked, will just write the uninstaller to some location, and then exit.
  ; Be sure to substitute the name of this script here.
 
  !system "$\"${NSISDIR}\makensis$\" /DINNER /DSUDOKU_VERSION=${SUDOKU_VERSION} /DGCCPATH=${GCCPATH} sudoku.nsi" = 0
 
  ; So now run that installer we just created as %TEMP%\tempinstaller.exe.  Since it
  ; calls quit the return value isn't zero.
 
  !system "$%TEMP%\tempinstaller.exe" = 2
 
  ; That will have written an uninstaller binary for us.  Now we sign it with your
  ; favourite code signing tool.
 
  !system "signtool sign /v /sha1 %CODESIGNSHA1% /t http://time.certum.pl/ $%TEMP%\${UNINST_NAME}" = 0
 
  ; Good.  Now we can carry on writing the real installer.
 
  OutFile "..\..\dist\sudoku-win32-${SUDOKU_VERSION}.exe"

  SetCompressor /SOLID /FINAL lzma
  SetCompressorDictSize 128

  RequestExecutionLevel admin
!endif

;--------------------------------
; Init function
;--------------------------------
Function .onInit
!ifdef INNER
 
  ; If INNER is defined, then we aren't supposed to do anything except write out
  ; the installer.  This is better than processing a command line option as it means
  ; this entire code path is not present in the final (real) installer.
 
  WriteUninstaller "$%TEMP%\${UNINST_NAME}"
  Quit  ; just bail out quickly when running the "inner" installer
!endif
FunctionEnd

;--------------------------------
; Variables
;--------------------------------

Var MUI_TEMP
Var STARTMENU_FOLDER

;--------------------------------
; Interface Settings
;--------------------------------

!define MUI_ABORTWARNING

;--------------------------------
; Pages
;--------------------------------

; Start Menu Folder Page Configuration

!define MUI_STARTMENUPAGE_REGISTRY_ROOT "${REG_ROOT}"
!define MUI_STARTMENUPAGE_REGISTRY_KEY "${REG_KEY}"
!define MUI_STARTMENUPAGE_REGISTRY_VALUENAME "Start Menu Folder"

!define MUI_HEADERIMAGE
!define MUI_HEADERIMAGE_RIGHT
!define MUI_HEADERIMAGE_BITMAP "sudoku.bmp"

!insertmacro MUI_PAGE_LICENSE "..\..\GPL.txt"
!insertmacro MUI_PAGE_DIRECTORY
!insertmacro MUI_PAGE_STARTMENU Application $STARTMENU_FOLDER
!insertmacro MUI_PAGE_INSTFILES
!insertmacro MUI_PAGE_FINISH

!ifdef INNER
!insertmacro MUI_UNPAGE_CONFIRM
!insertmacro MUI_UNPAGE_INSTFILES
!insertmacro MUI_UNPAGE_FINISH
!endif

;--------------------------------
; Languages
;--------------------------------

!insertmacro MUI_LANGUAGE "English"
!insertmacro MUI_LANGUAGE "German"

;--------------------------------
; Installer Sections
;--------------------------------

Section "Sudoku Section" SecSudoku

    ; Translations

    SetOutPath "$INSTDIR\i18n"

    File "..\..\i18n\sudoku.de_DE.qm"
    File "$%QTDIR%\translations\qt_de.qm"

    ; Main Files

    SetOutPath "$INSTDIR"

    File "..\..\gui\compile\release\sudoku.exe"
    File "..\..\GPL.html"
    File "..\..\ReleaseNotes.txt"
    File "$%QTDIR%\bin\QtCore4.dll"
    File "$%QTDIR%\bin\QtGui4.dll"
    File "$%QTDIR%\bin\QtXml4.dll"

    ; Runtimes

    File "${GCCPATH}\libgcc_s_*-1.dll"
    File "${GCCPATH}\libstdc++-6.dll"
    File "${GCCPATH}\libwinpthread-1.dll"

    ; Example fields

    SetOutPath "$%HOMEDRIVE%$%HOMEPATH%\.sternenwanderer.net\Sudoku\Examples"

    File "..\..\fields\*.*"

    ; Store installation folder

    WriteRegStr ${REG_ROOT} "${REG_KEY}" "" $INSTDIR

!ifndef INNER
    ; Write signed uninstaller

    SetOutPath "$INSTDIR"
    File $%TEMP%\${UNINST_NAME}
!endif

    ; Add uninstall entry to registry

    WriteRegStr ${REG_ROOT} "${REG_KEY_UNINSTALL}" "DisplayName" "Sudoku"
    WriteRegStr ${REG_ROOT} "${REG_KEY_UNINSTALL}" "DisplayIcon" "$\"$INSTDIR\sudoku.exe$\""
    WriteRegStr ${REG_ROOT} "${REG_KEY_UNINSTALL}" "Publisher" "sternenwanderer.net"
    WriteRegStr ${REG_ROOT} "${REG_KEY_UNINSTALL}" "UninstallString" "$\"$INSTDIR\${UNINST_NAME}$\""
    WriteRegStr ${REG_ROOT} "${REG_KEY_UNINSTALL}" "QuietUninstallString" "$\"$INSTDIR\${UNINST_NAME}$\" /S"
    WriteRegStr ${REG_ROOT} "${REG_KEY_UNINSTALL}" "DisplayVersion" "${SUDOKU_VERSION}"
    WriteRegStr ${REG_ROOT} "${REG_KEY_UNINSTALL}" "InstallLocation" "$\"$INSTDIR$\""
    WriteRegStr ${REG_ROOT} "${REG_KEY_UNINSTALL}" "URLInfoAbout" "http://sternenwanderer.net/programming/project/Sudoku.shtml"
    WriteRegDWORD ${REG_ROOT} "${REG_KEY_UNINSTALL}" "NoModify" 1
    WriteRegDWORD ${REG_ROOT} "${REG_KEY_UNINSTALL}" "NoRepair" 1

    ; Create shortcuts

    SetShellVarContext all
    SetOutPath "$INSTDIR"

    !insertmacro MUI_STARTMENU_WRITE_BEGIN Application
        CreateDirectory "$SMPROGRAMS\$STARTMENU_FOLDER"
        CreateShortCut "$SMPROGRAMS\$STARTMENU_FOLDER\Uninstall.lnk" "$INSTDIR\${UNINST_NAME}"
        CreateShortCut "$SMPROGRAMS\$STARTMENU_FOLDER\Sudoku.lnk" "$INSTDIR\sudoku.exe"
    !insertmacro MUI_STARTMENU_WRITE_END

    SetShellVarContext current

SectionEnd

;--------------------------------
; Descriptions
;--------------------------------

; Language strings

LangString DESC_SecSudoku ${LANG_ENGLISH} "Sudoku installation."
LangString DESC_SecSudoku ${LANG_GERMAN}  "Sudoku Installation."

; Assign language strings to sections

!insertmacro MUI_FUNCTION_DESCRIPTION_BEGIN
    !insertmacro MUI_DESCRIPTION_TEXT ${SecSudoku} $(DESC_SecSudoku)
!insertmacro MUI_FUNCTION_DESCRIPTION_END

;--------------------------------
; Uninstaller Section (only needed for inner invokation)
;--------------------------------

!ifdef INNER
Section "Uninstall"

    ; Remove everything

    RMDir /r /REBOOTOK "$INSTDIR"

    !insertmacro MUI_STARTMENU_GETFOLDER Application $MUI_TEMP

    SetShellVarContext all

    Delete "$SMPROGRAMS\$MUI_TEMP\Uninstall.lnk"
    Delete "$SMPROGRAMS\$MUI_TEMP\sudoku.lnk"

    ; Delete empty start menu parent diretories

    StrCpy $MUI_TEMP "$SMPROGRAMS\$MUI_TEMP"

    startMenuDeleteLoop:
	    ClearErrors
        RMDir $MUI_TEMP
        GetFullPathName $MUI_TEMP "$MUI_TEMP\.."

        IfErrors startMenuDeleteLoopDone

        StrCmp $MUI_TEMP $SMPROGRAMS startMenuDeleteLoopDone startMenuDeleteLoop
    startMenuDeleteLoopDone:

    SetShellVarContext current

    ; Cleanup registry

    DeleteRegKey ${REG_ROOT} "${REG_KEY}"
    DeleteRegKey /ifempty ${REG_ROOT} "${REG_KEY_BASE}"
    DeleteRegKey ${REG_ROOT} "${REG_KEY_UNINSTALL}"

SectionEnd
!endif
