/*
sudoku - Create and play sudoku

Copyright (C) 2007 Sven Heinzel <Sven.Heinzel@gmx.net>
 
This program is free software; you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation; either version 2 of the License, or
(at your option) any later version.
 
This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.
 
You should have received a copy of the GNU General Public License
along with this program; if not, write to the Free Software
Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
*/

#pragma once

#ifndef GENERATEOPTIONS_H
#define GENERATEOPTIONS_H

#include "sudoku/sudoku.h"
#include "sudoku/SolveOptions.h"

namespace Sudoku {

class GenerateOptions : public SolveOptions
{
public:
    enum Type {
        GenerateEmpty = 0,
        GenerateFields,
        FillFixedFields
    };

    GenerateOptions()
        : SolveOptions()
        , mType( GenerateFields )
        , mFixedFields( 27 )
        , mSymmetric( true )
    {
    }

    GenerateOptions( Type const  type, int const  fixedFields, bool const  symmetric, SolveOptions const  &rSolveOptions = SolveOptions() )
        : SolveOptions( rSolveOptions )
        , mType( type )
        , mFixedFields( ValidFixedField( fixedFields ) )
        , mSymmetric( symmetric )
    {
    }

    void SetType( Type const  type )
    {
        mType = type;
    }

    void SetFixedFields( int const  fixedFields )
    {
        mFixedFields = ValidFixedField( fixedFields );
    }

    void SetSymmetric( bool const  symmetric )
    {
        mSymmetric = symmetric;
    }

    int GetType() const
    {
        return mType;
    }

    int GetFixedFields() const
    {
        return mFixedFields;
    }

    static int GetFixedFieldsMin()
    {
        return 17;
    }

    static int GetFixedFieldsMax()
    {
        return 81 * 5 / 9;
    }

    bool IsSymmetric() const
    {
        return mSymmetric;
    }

private:
    static int ValidFixedField( int  fixedFields )
    {
        return std::max( GetFixedFieldsMin(), std::min( GetFixedFieldsMax(), fixedFields ) );
    }

    Type  mType;
    int   mFixedFields;
    bool  mSymmetric;
};

}

#endif
