/*
sudoku - Create and play sudoku

Copyright (C) 2007 Sven Heinzel <Sven.Heinzel@gmx.net>
 
This program is free software; you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation; either version 2 of the License, or
(at your option) any later version.
 
This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.
 
You should have received a copy of the GNU General Public License
along with this program; if not, write to the Free Software
Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
*/

#pragma once

#ifndef SUDOKU_GENERATOR_H
#define SUDOKU_GENERATOR_H

#include "sudoku/sudoku.h"
#include "sudoku/GenerateOptions.h"
#include "sudoku/Grid.h"

namespace Sudoku {

class Generator
{
public:
    Generator()
        : mGenerateOptions()
    {
    }

    Generator( GenerateOptions const  &rGenerateOptions )
        : mGenerateOptions( rGenerateOptions )
    {
    }

    ~Generator()
    {
    }

    inline void SetGenerateOptions( GenerateOptions const  &rGenerateOptions )
    {
        mGenerateOptions = rGenerateOptions;
    }

    inline GenerateOptions GetGenerateOptions() const
    {
        return mGenerateOptions;
    }

    bool Generate( Grid  &rGrid ) const;

    inline bool operator()( Grid  &rGrid ) const
    {
        return Generate( rGrid );
    }

private:
    class Instance
    {
    public:
        Instance( GenerateOptions const  &rGenerateOptions, Grid  &rGrid )
            : mGenerateOptions( rGenerateOptions )
            , mrGrid( rGrid )
            , mCount( rGrid . GetCount() )
        {
            Shuffle( mRowOrder );
            for( int  i = 0; i < mCount; ++i ) Shuffle( mColOrder[ i ] );
            Shuffle( mFieldOrder );
        }

        ~Instance()
        {
        }

        bool Generate();

    private:
        inline void Shuffle( std::vector< int >  &rData )
        {
            // Fill with numbers in order
            rData . resize( mCount );
            for( int  i = 0; i < mCount; ++i ) rData[ i ] = i;
            // Shuffle items
            random_shuffle( rData . begin(), rData . end() );
        }

        inline void Shuffle( std::vector< std::pair< int, int > >  &rData )
        {
            // Fill with numbers in order
            rData . resize( mCount * mCount );
            for( int  y = 0; y < mCount; ++y ) {
                for( int  x = 0; x < mCount; ++x ) {
                    rData[ y * mCount + x ] = std::make_pair( x, y );
                }
            }
            // Shuffle items
            random_shuffle( rData . begin(), rData . end() );
        }

        bool FillFields( int const  sIx = 0, int const  sIy = 0 );
        bool RemoveFields( int const  sI = 0 );
        bool RemoveField( int const  x, int const  y );
        void FixFields();

        GenerateOptions  mGenerateOptions;
        Grid             &mrGrid;
        int const        mCount;
        std::vector< int >  mRowOrder;
        std::vector< int >  mColOrder[ 64 ];
        std::vector< std::pair< int, int > >  mFieldOrder;
    };

    GenerateOptions  mGenerateOptions;
};

}

#endif
