/*
sudoku - Create and play sudoku

Copyright (C) 2013 Sven Heinzel <Sven.Heinzel@gmx.net>
 
This program is free software; you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation; either version 2 of the License, or
(at your option) any later version.
 
This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.
 
You should have received a copy of the GNU General Public License
along with this program; if not, write to the Free Software
Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
*/

#pragma once

#ifndef SUDOKU_LAYOUT_H
#define SUDOKU_LAYOUT_H

#include "sudoku/sudoku.h"

namespace Sudoku {

class Layout
{
public:
    Layout( int const  count = 9, bool const  randomLayout = false )
    {
        Resize( count, randomLayout );
    }

    Layout( Layout const  &rOther )
    {
        Copy( rOther );
    }

    ~Layout()
    {
    }

    void Resize( int const  count = 9, bool const  randomLayout = false );

    inline bool operator==( Layout const  &rOther ) const
    {
        if(    mCount     != rOther . mCount
            || mAvgWidth  != rOther . mAvgWidth
            || mAvgHeight != rOther . mAvgHeight ) return false;
        for( int  y = 0; y < mCount; ++y ) {
            for( int  x = 0; x < mCount; ++x ) {
                bool const  equal = mBoxIndex[ y ][ x ] == rOther . mBoxIndex[ y ][ x ];
                if( ! equal ) return false;
            }
        }
        return true;
    }

    inline bool operator!=( Layout const  &rOther ) const
    {
        return ! ( *this == rOther );
    }

    inline Layout & operator=( Layout const  &rOther )
    {
        if( this != &rOther ) {
            Copy( rOther );
        }
        return *this;
    }

    inline int BoxIndex( int const  x, int const  y ) const
    {
        if( ! CheckRange( x, y ) ) return -1;
        return mBoxIndex[ y ][ x ];
    }

    inline int GetCount() const
    {
        return mCount;
    }

private:
    inline bool CheckRange( int const  x, int const  y ) const
    {
        return ( 0 <= x && x < mCount ) && ( 0 <= y && y < mCount );
    }

    inline void Copy( Layout const  &rOther )
    {
        mCount     = rOther . mCount;
        mAvgWidth  = rOther . mAvgWidth;
        mAvgHeight = rOther . mAvgHeight;

        // Only copy filled entries
        for( int  y = 0; y < mCount; ++y ) {
            for( int  x = 0; x < mCount; ++x ) {
                mBoxIndex[ y ][ x ] = rOther . mBoxIndex[ y ][ x ];
            }
        }
    }

    int  mCount;
    int  mAvgWidth;
    int  mAvgHeight;
    int  mBoxIndex[ 64 ][ 64 ];
};

}

#endif
