/*
sudoku - Create and play sudoku

Copyright (C) 2007 Sven Heinzel <Sven.Heinzel@gmx.net>
 
This program is free software; you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation; either version 2 of the License, or
(at your option) any later version.
 
This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.
 
You should have received a copy of the GNU General Public License
along with this program; if not, write to the Free Software
Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
*/

#pragma once

#ifndef SOLVEOPTIONS_H
#define SOLVEOPTIONS_H

#include "sudoku/sudoku.h"

namespace Sudoku {

class SolveOptions
{
public:
    SolveOptions()
        : mMaxPairSolving( 2 )
        , mAllowBacktrack( false )
    {
    }

    SolveOptions( int const  maxPairSolving, bool const  allowBacktrack )
        : mMaxPairSolving( maxPairSolving )
        , mAllowBacktrack( allowBacktrack )
    {
    }

    static inline SolveOptions none()
    {
        return SolveOptions( 0, false );
    }

    static inline SolveOptions all()
    {
        // Max usefull pair solving is maxnum - 2.
        // Any greater value would mean the remaining number is unique.
        return SolveOptions( 7, true );
    }

    inline void SetMaxPairSolving( int const  maxPairSolving )
    {
        mMaxPairSolving = ValidMaxPairSolving( maxPairSolving );
    }

    inline void SetBacktrackAllowed( bool const  allowBacktrack )
    {
        mAllowBacktrack = allowBacktrack;
    }

    inline int GetMaxPairSolving() const
    {
        return mMaxPairSolving;
    }

    inline bool IsBacktrackAllowed() const
    {
        return mAllowBacktrack;
    }

private:
    static inline int ValidMaxPairSolving( int const  maxPairSolving )
    {
        return std::max( 0, std::min( 9, maxPairSolving ) );
    }

    int   mMaxPairSolving;
    bool  mAllowBacktrack;
};

}

#endif
