/*
sudoku - Create and play sudoku

Copyright (C) 2007 Sven Heinzel <Sven.Heinzel@gmx.net>
 
This program is free software; you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation; either version 2 of the License, or
(at your option) any later version.
 
This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.
 
You should have received a copy of the GNU General Public License
along with this program; if not, write to the Free Software
Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
*/

#pragma once

#ifndef SYSTEM_H
#define SYSTEM_H

#include "sudoku/sudoku.h"

namespace Sudoku {

class Timer
{
    CLASS_PRIVATE( Timer );
    CLASS_PRIVATE_COPY( Timer );

public:
    static uint64_t Nanoseconds() //!< Returns an unaligned timer value in nanoseconds
    {
        uint64_t  timer_val = 0;
        bool      ret = true;

#ifdef _WIN32
        static double  time_fact = 0.0;

        if ( time_fact == 0.0 )
        {
            uint64_t  time_freq = 0;

            // Get frequency only once

            if ( ret ) ret = QueryPerformanceFrequency( reinterpret_cast< LARGE_INTEGER* >( &time_freq ) );
            if ( ret ) time_fact = 1.0e9 / static_cast< double >( time_freq );
        }

        // Get counter

        if ( ret ) ret = QueryPerformanceCounter( reinterpret_cast< LARGE_INTEGER* >( &timer_val ) );

        // Convert to microseconds

        if ( ret ) timer_val = static_cast< uint64_t >( static_cast< double >( timer_val ) * time_fact );
#else
        timeval  time_tv;

        // Get time

        if ( ret ) ret = ! gettimeofday( &time_tv, 0 );

        // Convert to microseconds

        if ( ret ) timer_val = static_cast< uint64_t >( time_tv . tv_sec ) * static_cast< uint64_t >( 1000000000 ) + time_tv . tv_usec;
#endif

        // Exit

        return timer_val;
    }
};

}

#endif
