/*
sudoku - Create and play sudoku

Copyright (C) 2007 Sven Heinzel <Sven.Heinzel@gmx.net>
 
This program is free software; you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation; either version 2 of the License, or
(at your option) any later version.
 
This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.
 
You should have received a copy of the GNU General Public License
along with this program; if not, write to the Free Software
Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
*/

#ifndef SUDOKU_H
#define SUDOKU_H

/**
\mainpage libsudoku Reference Documentation

sudoku is a library for simple sudoku generating/solving functions

\section license_sec License

Copyright ©2009 by Sven Heinzel

Permission to use, copy, modify, and distribute this library and its
documentation under the terms of the GNU General Public License is
hereby granted. No representations are made about the suitability of this
software for any purpose. It is provided "as is" without express or implied
warranty. See the <A HREF="http://www.gnu.org/licenses/gpl-2.0.html">
GNU General Public License</A> for more details. 
*/

#if defined( _WIN32 ) || defined( _WIN64 )
#define WIN32_LEAN_AND_MEAN 1
#ifndef _WIN32_WINNT
#define _WIN32_WINNT 0x0500
#endif
#ifndef __MSVCRT__
#define __MSVCRT__
#endif
#ifndef __MSVCRT_VERSION__
#define __MSVCRT_VERSION__ 0x800
#endif
#endif

#ifndef NOMINMAX
#define NOMINMAX
#endif

#define CLASS_PRIVATE( x )  x(); ~x()
#define CLASS_PRIVATE_COPY( x )  x( x const & ); x & operator=( x const & )

#define _FILE_OFFSET_BITS 64

#include <stdio.h>

#ifdef __cplusplus
#include <iostream>
#include <iomanip>
#include <fstream>
#include <sstream>
#include <list>
#include <vector>
#include <map>
#include <algorithm>
#include <stdexcept>
#include <memory>
#endif

#include <errno.h>
#include <time.h>
#include <math.h>
#include <sys/types.h>
#include <sys/stat.h>

#if defined( _WIN32 ) || defined( _WIN64 )
#include <windows.h>
#include <wchar.h>
#include <malloc.h>
#include <sys/timeb.h>
#else
#include <sys/sysctl.h>
#include <stdlib.h>
#include <unistd.h>
#include <dirent.h>
#endif

#if !defined( _MSC_VER )
#include <sys/time.h>
#endif

#if defined( __APPLE__ )
#include <Carbon/Carbon.h> 
#include <CFString.h>
#endif 

#ifdef min
#undef min
#endif

#ifdef max
#undef max
#endif

#ifdef __cplusplus
extern "C"
{
#endif

#if defined( _MSC_VER )

// MSVC

#define __PRETTY_FUNCTION__  __FUNCSIG__

typedef unsigned __int8     uint8_t;
typedef signed __int8       int8_t;
typedef unsigned __int16    uint16_t;
typedef signed __int16      int16_t;
typedef unsigned __int32    uint32_t;
typedef signed __int32      int32_t;
typedef unsigned __int64    uint64_t;
typedef signed __int64      int64_t;

#else

// Standard

#include <stdint.h>

#endif

#ifdef __cplusplus
}
#endif

#define gettext_noop(x) x

#endif
