@echo off
rem Create documentation from the source code using doxygen

setlocal

rem Remove old documentation...
echo Removing old documentation...
rmdir /S /Q doc > NUL

rem Create new documentation...
mkdir doc
echo Creating new documentation...
for /F %%i in ('perl scripts\version.pl include\sudoku\sudoku_version.h short') do @set PROJECT_VERSION=%%i
doxygen scripts\sudoku.dox > doxygen.log
copy doxygen\html\*.* doc\html > NUL
echo Finished!

rem Create PDF...
echo Creating PDF...
copy doxygen\latex\*.* doc\latex > NUL
copy doxygen\sternenwanderer-dox.sty doc\latex > NUL
pushd doc\latex
pdflatex refman.tex > ..\..\doxygen.pdf.log 2> ..\..\doxygen.pdf.err
makeindex refman.idx >> ..\..\doxygen.pdf.log 2>> ..\..\doxygen.pdf.err
pdflatex refman.tex >> ..\..\doxygen.pdf.log 2>> ..\..\doxygen.pdf.err
pdflatex refman.tex >> ..\..\doxygen.pdf.log 2>> ..\..\doxygen.pdf.err
popd
echo Finished!

rem Compressing documentation...
echo Compressing documentation...
set COMPRESSION=-tzip -mx=9 -mfb=128 -mpass=10
pushd doc
7z a %COMPRESSION% -r libsudoku-%PROJECT_VERSION%-html.zip html > ..\doxygen.zip.log 2> ..\doxygen.zip.err
copy latex\refman.pdf libsudoku-%PROJECT_VERSION%.pdf > NUL
7z a %COMPRESSION% libsudoku-%PROJECT_VERSION%-pdf.zip libsudoku-%PROJECT_VERSION%.pdf >> ..\doxygen.zip.log 2>> ..\doxygen.zip.err
popd
echo Finished!

endlocal
