/*
sudoku - Create and play sudoku

Copyright (C) 2013 Sven Heinzel <Sven.Heinzel@gmx.net>
 
This program is free software; you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation; either version 2 of the License, or
(at your option) any later version.
 
This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.
 
You should have received a copy of the GNU General Public License
along with this program; if not, write to the Free Software
Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
*/

#include "sudoku/sudoku.h"
#include "sudoku/Layout.h"

namespace Sudoku {

void Layout::Resize( int const  count, bool const  randomLayout )
{
    // Keep count in range 4...64
    mCount = std::max( 4, std::min( count, 64 ) );

    // Calculate avergae width and height for one box for given count
    mAvgWidth = static_cast< int >( sqrt( static_cast< double >( mCount ) ) ); // Truncate
    mAvgHeight = mCount / mAvgWidth; // Also truncate

    // Now create layout
    if( randomLayout ) {
        // Random layout
        throw std::exception();
    } else {
        // Non random layouts must be rectangular
        mCount = mAvgWidth * mAvgHeight;
        for( int  y = 0; y < mCount; ++y ) {
            int const  y_offset = ( y / mAvgHeight ) * mAvgHeight;
            for( int  x = 0; x < mCount; ++x ) {
                mBoxIndex[ y ][ x ] = x / mAvgWidth + y_offset;
            }
        }
    }
}

}
