#!/usr/bin/perl

# sudoku - Create and play sudoku
# 
# Copyright (C) 2007 Sven Heinzel <Sven.Heinzel@gmx.net>
# 
# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 2 of the License, or
# (at your option) any later version.
# 
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
# 
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA

my $prefix = "SUDOKU";
my $version_major = 0;
my $version_minor = 0;
my $version_patch = 0;
my $version_build = 0;
my $version_extra = "";

# Read version header
open(FILE, $ARGV[0]);
@lines = <FILE>;
close(FILE);

foreach $line (@lines)
{
    if    ($line =~ m/^\s*#\s*define\s+($prefix)_MAJOR\s+\d+/o) { $line =~ s/^\s*#\s*define\s+\Q$prefix\E_MAJOR\s+(\d+)/$version_major = $1/ge; }
    elsif ($line =~ m/^\s*#\s*define\s+($prefix)_MINOR\s+\d+/o) { $line =~ s/^\s*#\s*define\s+\Q$prefix\E_MINOR\s+(\d+)/$version_minor = $1/ge; }
    elsif ($line =~ m/^\s*#\s*define\s+($prefix)_PATCH\s+\d+/o) { $line =~ s/^\s*#\s*define\s+\Q$prefix\E_PATCH\s+(\d+)/$version_patch = $1/ge; }
    elsif ($line =~ m/^\s*#\s*define\s+($prefix)_EXTRA\s+\S+/o) { $line =~ s/^\s*#\s*define\s+\Q$prefix\E_EXTRA\s+"(.+)"/$version_extra = $1/ge; }
}

# Read build
open(FILE, "svnversion|");
@lines = <FILE>;
close(FILE);

foreach $line (@lines)
{
    $version_build = sprintf( "%d", $line );
}

# Update resource file
open(RCFILE, $ARGV[ 1 ].".base");
@lines = <RCFILE>;
close(RCFILE);

open(RCFILE, ">".$ARGV[ 1 ]);
foreach $line (@lines)
{
    if    ($line =~ m/^\s+FILEVERSION\s+\d+/)          { printf RCFILE " FILEVERSION %d,%d,%d,%d\n", $version_major, $version_minor, $version_patch, $version_build; }
    elsif ($line =~ m/^\s+PRODUCTVERSION\s+\d+/)       { printf RCFILE " PRODUCTVERSION %d,%d,%d,%d\n", $version_major, $version_minor, $version_patch, $version_build; }
    elsif ($line =~ m/^\s+VALUE\s+\"FileVersion\"/)    { printf RCFILE "            VALUE \"FileVersion\", \"%d.%d.%d.%d\\0\"\n", $version_major, $version_minor, $version_patch, $version_build; }
    elsif ($line =~ m/^\s+VALUE\s+\"ProductVersion\"/) { printf RCFILE "            VALUE \"ProductVersion\", \"%d.%d.%d.%d\\0\"\n", $version_major, $version_minor, $version_patch, $version_build; }
    elsif ($line =~ m/^\s+VALUE\s+\"SpecialBuild\"/)   { printf RCFILE "            VALUE \"SpecialBuild\", \"%s\\0\"\n", $version_extra; }
    else                                               { print  RCFILE $line; }
}
close(RCFILE);
